unit RSSD1289;

{
Copyright (C) 2016 - Ronald Daleske

Licence
=======

 LGPLv2.1 with static linking exception (See COPYING.modifiedLGPL.txt)

}


{$mode objfpc}{$H+}

interface

uses
  Threads,
  Platform,
  RGPIO;


const



  // Festwerte fuer Farben
  cl_White    : Word = $FFFF;
  cl_Black    : Word = $0000;
  cl_Red      : Word = $F800;
  cl_Green    : Word = $0400;
  cl_Blue     : Word = $001F;
  cl_Yellow   : Word = $FFE0;
  cl_Purple   : Word = $8010;
  cl_Aqua     : Word = $07FF;
  cl_Gray     : Word = $8410;
  cl_DarkGray : Word = $8410;  // !!! Anpassen
  cl_Silver   : Word = $C618;
  cl_Maroon   : Word = $8000;
  cl_Olive    : Word = $8400;
  cl_Lime     : Word = $07E0;
  cl_Teal     : Word = $0410;
  cl_Navy     : Word = $0010;
  cl_Fuchsia  : Word = $F81F;


  const
{$I KAEFER1.INC}
{$I KAEFER2.INC}



procedure Init_SSD1289;
procedure clear_Screen(Pixel_Color : Word);
procedure drawVertikalLine_Quer(Pixel_Color, Pos_x, Pos_y1, Pos_y2 : Word);


procedure Write_Data_Word(WD_Word : Word);
procedure Write_Data_Low_Byte(WD_Byte : Byte);
procedure draw_2Picrure;



implementation






{
GPIO4  -> SERIAL_DATA
GPIO17 -> SHIFT_CLOCK
GPIO18 -> LATCH_CLOCK

// Data/Command
GPIO22 -> LCD_RS

// Write Low-Aktiv
GPIO23 -> LCD_WR

// Chip-Select Low-Aktiv
GPIO24 -> LCD_CS

// Reset Low-Aktiv
GPIO27 -> LCD_RESET

}




procedure Init_Ground;
begin

  // Initializierung
  GPIO_OUT_Init(GPIO4);
  GPIO_OUT_Init(GPIO17);
  GPIO_OUT_Init(GPIO18);

  GPIO_OUT_Init(GPIO22);
  GPIO_OUT_Init(GPIO23);
  GPIO_OUT_Init(GPIO24);
  GPIO_OUT_Init(GPIO27);

  // Ground-State
  GPIO4_Clear;
  GPIO17_Clear;
  GPIO18_Clear;


  // Daten/Kommandos
  // GPIO24 -> LCD_RS
  GPIO24_Clear;

  // Chip-Select Low-Aktiv
  // GPIO22 -> LCD_CS
  GPIO22_Set;

  // Write Low-Aktiv
  // GPIO23 -> LCD_WR
  GPIO23_Set;

  // Reset Low-Aktiv
  // GPIO27 -> LCD_RESET
  GPIO27_Set;

end; { procedure Init_Ground }




{ 111111111111111111111111111111111111111111111111111 }
{ *************************************************** }
{ *******         PIN-Routins for LCD         ******* }
{ *************************************************** }


// LCD_RS = 1 -> Data
procedure LCD_RS_Data;
begin
  GPIO24_Set;
end;


// LCD_RS = 0 -> Command
procedure LCD_RS_Command;
begin
  GPIO24_Clear;
end;



// Chip-Select Low-Aktiv = 1 -> inactive
procedure LCD_CS_Set_inactive;
begin
  GPIO22_Set;
end;



// Chip-Select Low-Aktiv = 0 -> inactive
procedure LCD_CS_Clear_active;
begin
  GPIO22_Clear;
end;



// Write Low-Aktiv
// GPIO23 -> LCD_WR
procedure LCD_WR_Set_inactive;
begin
  GPIO23_Set;
end;



// Write Low-Aktiv
// GPIO23 -> LCD_WR
procedure LCD_WR_Clear_active;
begin
  GPIO23_Clear;
end;



// Reset Low-Aktiv
// GPIO27 -> LCD_RESET
procedure LCD_RESET_Set_inactive;
begin
  GPIO27_Set;
end;



// Reset Low-Aktiv
// GPIO27 -> LCD_RESET
procedure LCD_RESET_Clear_active;
begin
  GPIO27_Clear;
end;




{ 222222222222222222222222222222222222222222222222222 }
{ *************************************************** }
{ *******    Data-Register-Routins for LCD    ******* }
{ *************************************************** }


procedure Write_Data_Word(WD_Word : Word);
var
  R_Word_Latch1, R_Word_Latch2 : Word;
  R_counter : Byte;
begin

  R_Word_Latch1:=WD_Word;

  for R_counter:=1 to 16 do begin

    GPIO17_Clear;

    R_Word_Latch2:=R_Word_Latch1 and %0000000000000001;

    if R_Word_Latch2=0 then
      GPIO4_Clear    // Data-Bit=0
    else
      GPIO4_Set;     // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    GPIO17_Set;

    R_Word_Latch1:=R_Word_Latch1 shr 1;

  end; { for R_counter:=1 to 16 do }

  GPIO18_Set;
  NanosecondDelay(100);

  GPIO4_Clear;
  GPIO17_Clear;
  GPIO18_Clear;

end; { procedure Write_Data_Word }







procedure Write_Data_Low_Byte(WD_Byte : Byte);
var
  R_Byte_Latch1, R_Byte_Latch2 : Byte;
  R_counter : Byte;
begin

  R_Byte_Latch1:=WD_Byte;

  for R_counter:=1 to 8 do begin

    GPIO17_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      GPIO4_Clear    // Data-Bit=0
    else
      GPIO4_Set;     // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    GPIO17_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  GPIO4_Clear;    // Data-Bit=0

  // 8 Byte -> 0
  for R_counter:=1 to 8 do begin

    GPIO17_Clear;

    NanosecondDelay(10);

    // Shift IN Data-Bit
    GPIO17_Set;

    NanosecondDelay(10);

  end; { for R_counter:=1 to 8 do }

  GPIO18_Set;
  NanosecondDelay(100);

  GPIO4_Clear;
  GPIO17_Clear;
  GPIO18_Clear;

end; { procedure Write_Data_Low_Byte }




procedure Write_Data_Hight_Low_Byte(Hight_Byte, Low_Byte : Byte);
var
  R_Byte_Latch1, R_Byte_Latch2 : Byte;
  R_counter : Byte;
begin

  R_Byte_Latch1:=Low_Byte;

  for R_counter:=1 to 8 do begin

    GPIO17_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      GPIO4_Clear    // Data-Bit=0
    else
      GPIO4_Set;     // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    GPIO17_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  R_Byte_Latch1:=Hight_Byte;

  for R_counter:=1 to 8 do begin

    GPIO17_Clear;

    R_Byte_Latch2:=R_Byte_Latch1 and %00000001;

    if R_Byte_Latch2=0 then
      GPIO4_Clear    // Data-Bit=0
    else
      GPIO4_Set;     // Data-Bit=1

    NanosecondDelay(10);

    // Shift IN Data-Bit
    GPIO17_Set;

    R_Byte_Latch1:=R_Byte_Latch1 shr 1;

  end; { for R_counter:=1 to 8 do }

  GPIO18_Set;
  NanosecondDelay(100);

  GPIO4_Clear;
  GPIO17_Clear;
  GPIO18_Clear;

end; { procedure Write_Data_Hight_Low_Byte }




{ 333333333333333333333333333333333333333333333333333 }
{ *************************************************** }
{ *******        Basic-Routins for LCD        ******* }
{ *************************************************** }



procedure LCD_Write_COM(COM_Byte : Byte);
begin

  // Data/Command
  // LCD_RS = 0 -> Command
  LCD_RS_Command;

  NanosecondDelay(10);

  // DB8->DB15 = 0
  // DB0->DB7
  Write_Data_Low_Byte(COM_Byte);

  LCD_WR_Clear_active;
  NanosecondDelay(20);
  LCD_WR_Set_inactive;

end; { procedure LCD_Write_COM }




procedure LCD_Write_DATA(DATA_Byte_High, DATA_Byte_Low : Byte);
begin

  LCD_RS_Data;

  Write_Data_Hight_Low_Byte(DATA_Byte_High, DATA_Byte_Low);

  LCD_WR_Clear_active;
  NanosecondDelay(20);
  LCD_WR_Set_inactive;

end; { procedure LCD_Write_DATA }






procedure LCD_Write_DATA_Word(DATA_Word : Word);
begin

  LCD_RS_Data;

  Write_Data_Word(DATA_Word);

  LCD_WR_Clear_active;
  NanosecondDelay(20);
  LCD_WR_Set_inactive;

end; { procedure LCD_Write_DATA_Word }






procedure LCD_Write_COM_DATA(COM_Byte, DATA_Byte_High, DATA_Byte_Low : Byte);
begin

  LCD_Write_COM(COM_Byte);
  LCD_Write_DATA(DATA_Byte_High, DATA_Byte_Low);

end; { procedure LCD_Write_COM_DATA }





{ *************************************************** }
{ *******       Service-Routins for LCD       ******* }
{ *************************************************** }



procedure Init_SSD1289;
begin

  // send RESET to LCD
  LCD_RESET_Set_inactive;
  MicrosecondDelay(1);
  LCD_RESET_Clear_active;
  MillisecondDelay(15);
  LCD_RESET_Set_inactive;

  // Send INIT-Data

  MicrosecondDelay(1);

  // Chip-Select Low-Aktiv
  LCD_CS_Clear_active;

  // Einschalten des Oszillators
  LCD_Write_COM_DATA($00,$00,$01);

  //Power Control (1)
  LCD_Write_COM_DATA($03,$A8,$A4);

  //Power Control (2)
  LCD_Write_COM_DATA($0C,$00,$00);

  //Power Control (3)
  LCD_Write_COM_DATA($0D,$08,$0C);

  //Power Control (4)
  LCD_Write_COM_DATA($0E,$2B,$00);

  //Power Control (5)
  LCD_Write_COM_DATA($1E,$00,$B7);

  //Driver Output Control RL=0, REV=1, BGR=1, TB=1
  LCD_Write_COM_DATA($01,$2B,$3F);

  //Restore VSYNC mode from low power state
  LCD_Write_COM_DATA($02,$06,$00);

  //Sleep mode cancel
  LCD_Write_COM_DATA($10,$00,$00);

  //Entry Mode
  // DFM   0x4000 = 262L color
  // DFM   0x6000 = 65K color
  // AM    0x0000 = horizontal display
  // AM    0x0008 = Vertical display
  // ID[0] 0x0000 = horizontal decrement
  // ID[0] 0x0010 = horizontal increment
  // ID[1] 0x0000 = Vertical decrement
  // ID[1] 0x0020 = Vertical increment
  LCD_Write_COM_DATA($11,$60,$70);

  // Compare register
  LCD_Write_COM_DATA($05,$00,$00);

  // Compare register
  LCD_Write_COM_DATA($06,$00,$00);

  // Horizontal and Vertical porch are for DOTCLK mode operation
  // Horizontal Porch
  LCD_Write_COM_DATA($16,$EF,$1C);

  // Vertical Porch
  LCD_Write_COM_DATA($17,$00,$03);

  // Display Control
  // D1 0x0000 = display off
  // D1 0x0002 = display on
  // D0 0x0000 = internal display halt
  // D0 0x0001 = internal display operate
  LCD_Write_COM_DATA($07,$02,$33);

  // Frame cycle control
  LCD_Write_COM_DATA($0B,$00,$00);

  // Gate Scan Position
  LCD_Write_COM_DATA($0F,$00,$00);

  // Vertical Scroll Control
  LCD_Write_COM_DATA($41,$00,$00);
  LCD_Write_COM_DATA($42,$00,$00);

  // 1st Screen driving position
  // Start position. 0
  LCD_Write_COM_DATA($48,$00,$00);
  // End position.   319
  LCD_Write_COM_DATA($49,$01,$3F);



  // Source RAM address
  //Horizontal RAM address position start/end setup
  //dec 239
  //dec 0, i.e. horizontal ranges from 0 -> 239
  //POR value is 0xEF00 anyway. This address must be set before RAM write
  LCD_Write_COM_DATA($44,$EF,$00);

  //Vertical RAM address start position setting
  //0x0000 = dec 0
  LCD_Write_COM_DATA($45,$00,$00);

  //Vertical RAM address end position setting (0x013F = dec 319)
  LCD_Write_COM_DATA($46,$01,$3F);


  // 2nd Screen driving position
  // Start position. 0
  LCD_Write_COM_DATA($4A,$00,$00);
  // End position.   0
  LCD_Write_COM_DATA($4B,$00,$00);


  //gamma control
  LCD_Write_COM_DATA($30,$07,$07);
  LCD_Write_COM_DATA($31,$02,$04);
  LCD_Write_COM_DATA($32,$02,$04);
  LCD_Write_COM_DATA($33,$05,$02);
  LCD_Write_COM_DATA($34,$05,$07);
  LCD_Write_COM_DATA($35,$02,$04);
  LCD_Write_COM_DATA($36,$02,$04);
  LCD_Write_COM_DATA($37,$05,$02);
  LCD_Write_COM_DATA($3A,$03,$02);
  LCD_Write_COM_DATA($3B,$03,$02);


  LCD_Write_COM_DATA($23,$00,$00);
  LCD_Write_COM_DATA($24,$00,$00);
  LCD_Write_COM_DATA($25,$80,$00);
  LCD_Write_COM_DATA($4F,$00,$00);
  LCD_Write_COM_DATA($4E,$00,$00);
  LCD_Write_COM($22);


  // Chip-Select Low-Aktiv
  LCD_CS_Set_inactive;

end; { procedure Init_SSD1289 }





procedure LCD_Set_Address_Landscape(x1,y1,x2,y2 : Word);
var
  b1 : Byte;
  b2 : Byte;
  w1 : Word;
begin

  w1:=x1;
  x1:=y1;
  y1:=w1;

  w1:=x2;
  x2:=y2;
  y2:=w1;

  y1:=319-y1;
  y2:=319-y2;

  w1:=y1;
  y1:=y2;
  y2:=w1;

  b1:=x1 and $FF;
  b2:=x2 and $FF;
  Lcd_Write_Com_Data($44,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($45,b2,b1);

  b1:=y2 and $FF;
  b2:=y2 shr 8;
  Lcd_Write_Com_Data($46,b2,b1);

  b1:=x1 and $FF;
  b2:=x1 shr 8;
  Lcd_Write_Com_Data($4E,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($4F,b2,b1);

  Lcd_Write_Com($22);

end; { procedure LCD_Set_Address_Landscape }





procedure LCD_Set_Address_Portrait(x1,y1,x2,y2 : Word);
var
  b1 : Byte;
  b2 : Byte;
begin

  b1:=x1 and $FF;
  b2:=x2 and $FF;
  Lcd_Write_Com_Data($44,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($45,b2,b1);

  b1:=y2 and $FF;
  b2:=y2 shr 8;
  Lcd_Write_Com_Data($46,b2,b1);

  b1:=x1 and $FF;
  b2:=x1 shr 8;
  Lcd_Write_Com_Data($4E,b2,b1);

  b1:=y1 and $FF;
  b2:=y1 shr 8;
  Lcd_Write_Com_Data($4F,b2,b1);

  Lcd_Write_Com($22);

end; { procedure LCD_Set_Address_Portrait }







procedure clear_Screen(Pixel_Color : Word);
var
  b1, b2 : Word;
begin

  // Chip-Select Low-Aktiv
  LCD_CS_Clear_active;

  LCD_Set_Address_Landscape(0,0,319,239);

  // Data/Command
  // LCD_RS = 1 -> Data
  LCD_RS_Data;

  Write_DATA_Word(Pixel_Color);

  // Write Low-Aktiv
  LCD_WR_Clear_active;

  for b1:=1 to 320 do begin

    for b2:=1 to 240 do begin

      NanosecondDelay(5);

      // Write Low-Aktiv
      // GPIO23 -> LCD_WR
      LCD_WR_Set_inactive;

      NanosecondDelay(10);

      // Write Low-Aktiv
      LCD_WR_Clear_active;

    end; { for b2:=1 to 240 do }

  end; { for b1:=1 to 320 do }

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_Set_inactive;

  NanosecondDelay(10);

  // Chip-Select Low-Aktiv
  LCD_CS_Set_inactive;

end; { procedure clear_Screen }








procedure drawVertikalLine_Quer(Pixel_Color, Pos_x, Pos_y1, Pos_y2 : Word);
var
  Line : Word;
begin

  // Chip-Select Low-Aktiv
  LCD_CS_Clear_active;

  NanosecondDelay(10);

  LCD_Set_Address_Landscape(Pos_x,Pos_y1,Pos_x,Pos_y2);

  // Data/Command
  // LCD_RS = 1 -> Data
  LCD_RS_Data;

  NanosecondDelay(10);

  Write_Data_Word(Pixel_Color);

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_Clear_active;

  for Line:=Pos_y1 to Pos_y2 do begin

    NanosecondDelay(5);

    // Write Low-Aktiv
    LCD_WR_Set_inactive;

    NanosecondDelay(10);

    // Write Low-Aktiv
    LCD_WR_Clear_active;

  end;

  NanosecondDelay(10);

  // Write Low-Aktiv
  LCD_WR_Set_inactive;

  NanosecondDelay(10);

  // Chip-Select Low-Aktiv
  LCD_CS_Set_inactive;

end; { procedure drawVertikalLine_Quer }











procedure draw_2Picrure;
var
  w1 : Word;
  w2 : Word;
  w3 : Word;
  w4 : Word;
  Zeiger : Word;
  FarbWert : Word;
begin


  Zeiger:=0;
  w1:=KAEFER1[Zeiger];
  DEC(w1);
  INC(Zeiger);
  w2:=KAEFER1[Zeiger];
  DEC(w2);
  INC(Zeiger);

  LCD_CS_Clear_active;

  LCD_Set_Address_Portrait(0,0,w1,w2);

  LCD_RS_Data;

  for w3:=0 to w1 do begin

    for w4:=0 to w2 do begin

      FarbWert:=KAEFER1[Zeiger];
      INC(Zeiger);

      LCD_WR_Set_inactive;

      Write_DATA_Word(FarbWert);

      LCD_WR_Clear_active;

    end;

  end;

  NanosecondDelay(20);
  LCD_WR_Set_inactive;


  Zeiger:=0;
  w1:=KAEFER2[Zeiger];
  DEC(w1);
  INC(Zeiger);
  w2:=KAEFER2[Zeiger];
  DEC(w2);
  INC(Zeiger);

  w4:=w2+160;

  LCD_CS_Clear_active;

  LCD_Set_Address_Portrait(0,160,w1,w4);

  LCD_RS_Data;

  for w3:=0 to w1 do begin

    for w4:=0 to w2 do begin

      FarbWert:=KAEFER2[Zeiger];
      INC(Zeiger);

      LCD_WR_Set_inactive;

      Write_DATA_Word(FarbWert);

      LCD_WR_Clear_active;

    end;

  end;

  NanosecondDelay(20);
  LCD_WR_Set_inactive;

  NanosecondDelay(20);
  LCD_CS_Set_inactive;

end;






{==============================================================================}
{==============================================================================}

initialization

  Init_Ground;



{==============================================================================}

finalization
 {Nothing}

{==============================================================================}
{==============================================================================}

end.









end.

